@@ -5,7 +5,7 @@
  *
  * Author: Philip Withnall <pwithnall@endlessos.org>
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 /**
@@ -33,13 +33,18 @@
  * into the job will not be modified.
  *
  * Internally, the #GsPluginClass.refine_async() functions are called on all
- * the plugins in series, and in series with a call to
- * gs_odrs_provider_refine_async(). Once all of those calls are finished,
+ * the plugins in series, and in series with calls to
+ * gs_odrs_provider_refine_async() and gs_rewrite_resources_async().
+ * Once all of those calls are finished,
  * zero or more recursive calls to run_refine_internal_async() are made in
  * parallel to do a similar refine process on the addons, runtime and related
  * components for all the components in the input #GsAppList. The refine job is
  * complete once all these recursive calls complete.
  *
+ * The call to gs_rewrite_resources_async() will rewrite the CSS of apps to
+ * refer to locally cached resources, rather than HTTP/HTTPS URIs for images
+ * (for example).
+ *
  * FIXME: Ideally, the #GsPluginClass.refine_async() calls would happen in
  * parallel, but this cannot be the case until the results of the refine_async()
  * call in one plugin don’t depend on the results of refine_async() in another.
@@ -49,15 +54,16 @@
  *                                    run_async()
  *                                         |
  *                                         v
- *           /-----------------------+-------------+----------------\
- *           |                       |             |                |
- * plugin->refine_async()            |             |                |
- *           v             plugin->refine_async()  |                |
- *           |                       v             …                |
- *           |                       |             v  gs_odrs_provider_refine_async()
- *           |                       |             |                v
- *           |                       |             |                |
- *           \-----------------------+-------------+----------------/
+ *           /-----------------------+-------------+----------------+------------------------------\
+ *           |                       |             |                |                              |
+ * plugin->refine_async()            |             |                |                              |
+ *           v             plugin->refine_async()  |                |                              |
+ *           |                       v             …                |                              |
+ *           |                       |             v  gs_odrs_provider_refine_async()              |
+ *           |                       |             |                v                  gs_rewrite_resources_async()
+ *           |                       |             |                |                              v
+ *           |                       |             |                |                              |
+ *           \-----------------------+-------------+----------------+------------------------------/
  *                                         |
  *                            finish_refine_internal_op()
  *                                         |
@@ -87,8 +93,10 @@
 #include "gs-app-private.h"
 #include "gs-app-list-private.h"
 #include "gs-enums.h"
+#include "gs-plugin-private.h"
 #include "gs-plugin-job-private.h"
 #include "gs-plugin-job-refine.h"
+#include "gs-profiler.h"
 #include "gs-utils.h"
 
 struct _GsPluginJobRefine
@@ -101,6 +109,10 @@ struct _GsPluginJobRefine
 
 	/* Output data. */
 	GsAppList *result_list;  /* (owned) (nullable) */
+
+#ifdef HAVE_SYSPROF
+	gint64 begin_time_nsec;
+#endif
 };
 
 G_DEFINE_TYPE (GsPluginJobRefine, gs_plugin_job_refine, GS_TYPE_PLUGIN_JOB)
@@ -218,6 +230,9 @@ static void plugin_refine_cb (GObject      *source_object,
 static void odrs_provider_refine_cb (GObject      *source_object,
                                      GAsyncResult *result,
                                      gpointer      user_data);
+static void rewrite_resources_cb (GObject      *source_object,
+                                  GAsyncResult *result,
+                                  gpointer      user_data);
 static void finish_refine_internal_op (GTask  *task,
                                        GError *error);
 static void recursive_internal_refine_cb (GObject      *source_object,
@@ -239,6 +254,11 @@ typedef struct {
 	guint n_pending_ops;
 	guint n_pending_recursions;
 	guint next_plugin_index;
+	guint next_plugin_order;
+
+#ifdef HAVE_SYSPROF
+	gint64 plugin_begin_time_nsec;
+#endif
 
 	/* Output data. */
 	GError *error;  /* (nullable) (owned) */
@@ -275,6 +295,8 @@ run_refine_internal_async (GsPluginJobRefine   *self,
 	g_autoptr(GTask) task = NULL;
 	RefineInternalData *data;
 	g_autoptr(RefineInternalData) data_owned = NULL;
+	gboolean anything_ran = FALSE;
+	g_autoptr(GError) local_error = NULL;
 
 	task = g_task_new (self, cancellable, callback, user_data);
 	g_task_set_source_tag (task, run_refine_internal_async);
@@ -283,12 +305,16 @@ run_refine_internal_async (GsPluginJobRefine   *self,
 	data->plugin_loader = g_object_ref (plugin_loader);
 	data->list = g_object_ref (list);
 	data->flags = flags;
+#ifdef HAVE_SYSPROF
+	data->plugin_begin_time_nsec = SYSPROF_CAPTURE_CURRENT_TIME;
+#endif
 	g_task_set_task_data (task, g_steal_pointer (&data_owned), (GDestroyNotify) refine_internal_data_free);
 
-	/* try to adopt each application with a plugin */
+	/* try to adopt each app with a plugin */
 	gs_plugin_loader_run_adopt (plugin_loader, list);
 
 	data->n_pending_ops = 0;
+	data->next_plugin_order = 0;
 
 	/* run each plugin
 	 *
@@ -306,11 +332,25 @@ run_refine_internal_async (GsPluginJobRefine   *self,
 		GsPlugin *plugin = g_ptr_array_index (plugins, i);
 		GsPluginClass *plugin_class = GS_PLUGIN_GET_CLASS (plugin);
 
+		if (gs_plugin_get_order (plugin) > data->next_plugin_order) {
+			if (!anything_ran)
+				data->next_plugin_order = gs_plugin_get_order (plugin);
+			else
+				return;
+		}
+
 		if (!gs_plugin_get_enabled (plugin))
 			continue;
 		if (plugin_class->refine_async == NULL)
 			continue;
 
+		/* at least one plugin supports this vfunc */
+		anything_ran = TRUE;
+
+		/* Handle cancellation */
+		if (g_cancellable_set_error_if_cancelled (cancellable, &local_error))
+			break;
+
 		/* FIXME: The next refine_async() call is made in
 		 * finish_refine_internal_op(). */
 		data->next_plugin_index = i + 1;
@@ -319,14 +359,13 @@ run_refine_internal_async (GsPluginJobRefine   *self,
 		data->n_pending_ops++;
 		plugin_class->refine_async (plugin, list, flags,
 					    cancellable, plugin_refine_cb, g_object_ref (task));
-
-		/* FIXME: The next refine_async() call is made in
-		 * finish_refine_internal_op(). */
-		return;
 	}
 
+	if (!anything_ran)
+		g_debug ("no plugin could handle refining apps");
+
 	data->n_pending_ops++;
-	finish_refine_internal_op (task, NULL);
+	finish_refine_internal_op (task, g_steal_pointer (&local_error));
 }
 
 static void
@@ -338,10 +377,25 @@ plugin_refine_cb (GObject      *source_object,
 	g_autoptr(GTask) task = g_steal_pointer (&user_data);
 	GsPluginClass *plugin_class = GS_PLUGIN_GET_CLASS (plugin);
 	g_autoptr(GError) local_error = NULL;
+#ifdef HAVE_SYSPROF
+	GsPluginJobRefine *self = g_task_get_source_object (task);
+	RefineInternalData *data = g_task_get_task_data (task);
+#endif
 
-	if (!plugin_class->refine_finish (plugin, result, &local_error)) {
-		finish_refine_internal_op (task, g_steal_pointer (&local_error));
-		return;
+	GS_PROFILER_ADD_MARK_TAKE (PluginJobRefine,
+				   data->plugin_begin_time_nsec,
+				   g_strdup_printf ("%s:%s",
+						    G_OBJECT_TYPE_NAME (self),
+						    gs_plugin_get_name (plugin)),
+				   NULL);
+
+	if (!plugin_class->refine_finish (plugin, result, &local_error) &&
+	    !g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+	    !g_error_matches (local_error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED)) {
+		g_debug ("plugin '%s' failed to refine apps: %s",
+			 gs_plugin_get_name (plugin),
+			 local_error->message);
+		g_clear_error (&local_error);
 	}
 
 	gs_plugin_status_update (plugin, NULL, GS_PLUGIN_STATUS_FINISHED);
@@ -358,7 +412,31 @@ odrs_provider_refine_cb (GObject      *source_object,
 	g_autoptr(GTask) task = g_steal_pointer (&user_data);
 	g_autoptr(GError) local_error = NULL;
 
-	gs_odrs_provider_refine_finish (odrs_provider, result, &local_error);
+	if (!gs_odrs_provider_refine_finish (odrs_provider, result, &local_error) &&
+	    !g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+	    !g_error_matches (local_error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED)) {
+		g_debug ("ODRS provider failed to refine apps: %s",
+			 local_error->message);
+		g_clear_error (&local_error);
+	}
+	finish_refine_internal_op (task, g_steal_pointer (&local_error));
+}
+
+static void
+rewrite_resources_cb (GObject      *source_object,
+                      GAsyncResult *result,
+                      gpointer      user_data)
+{
+	g_autoptr(GTask) task = g_steal_pointer (&user_data);
+	g_autoptr(GError) local_error = NULL;
+
+	if (!gs_rewrite_resources_finish (result, &local_error) &&
+	    !g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+	    !g_error_matches (local_error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED)) {
+		g_debug ("Rewriting resources failed when refine apps: %s",
+			 local_error->message);
+		g_clear_error (&local_error);
+	}
 	finish_refine_internal_op (task, g_steal_pointer (&local_error));
 }
 
@@ -377,6 +455,7 @@ finish_refine_internal_op (GTask  *task,
 	GsOdrsProvider *odrs_provider;
 	GsOdrsProviderRefineFlags odrs_refine_flags = 0;
 	GPtrArray *plugins;  /* (element-type GsPlugin) */
+	gboolean anything_ran = FALSE;
 
 	if (data->error == NULL && error_owned != NULL) {
 		data->error = g_steal_pointer (&error_owned);
@@ -387,16 +466,39 @@ finish_refine_internal_op (GTask  *task,
 	g_assert (data->n_pending_ops > 0);
 	data->n_pending_ops--;
 
+#ifdef HAVE_SYSPROF
+	data->plugin_begin_time_nsec = SYSPROF_CAPTURE_CURRENT_TIME;
+#endif
+
+	if (data->n_pending_ops > 0)
+		return;
+
+	/* We reach this line after all plugins of a certain order ran, and now
+	 * we need to run the next set of plugins. */
+	data->next_plugin_order++;
+
 	plugins = gs_plugin_loader_get_plugins (plugin_loader);
 
 	for (guint i = data->next_plugin_index; i < plugins->len; i++) {
 		GsPlugin *plugin = g_ptr_array_index (plugins, i);
 		GsPluginClass *plugin_class = GS_PLUGIN_GET_CLASS (plugin);
 
+		if (gs_plugin_get_order (plugin) > data->next_plugin_order) {
+			if (!anything_ran)
+				data->next_plugin_order = gs_plugin_get_order (plugin);
+			else
+				return;
+		}
+
 		if (!gs_plugin_get_enabled (plugin))
 			continue;
 		if (plugin_class->refine_async == NULL)
 			continue;
+		if (gs_plugin_get_order (plugin) < data->next_plugin_order)
+			continue;
+
+		/* at least one plugin supports this vfunc */
+		anything_ran = TRUE;
 
 		/* FIXME: The next refine_async() call is made in
 		 * finish_refine_internal_op(). */
@@ -406,14 +508,10 @@ finish_refine_internal_op (GTask  *task,
 		data->n_pending_ops++;
 		plugin_class->refine_async (plugin, list, flags,
 					    cancellable, plugin_refine_cb, g_object_ref (task));
-
-		/* FIXME: The next refine_async() call is made in
-		 * finish_refine_internal_op(). */
-		return;
 	}
 
 	if (data->next_plugin_index == plugins->len) {
-		/* Avoid the ODRS refine being run multiple times. */
+		/* Avoid the ODRS and rewrite refines being run multiple times. */
 		data->next_plugin_index++;
 
 		/* Add ODRS data if needed */
@@ -430,13 +528,18 @@ finish_refine_internal_op (GTask  *task,
 			gs_odrs_provider_refine_async (odrs_provider, list, odrs_refine_flags,
 						       cancellable, odrs_provider_refine_cb, g_object_ref (task));
 		}
+
+		/* Rewrite app CSS if needed. */
+		data->n_pending_ops++;
+		gs_rewrite_resources_async (list, cancellable, rewrite_resources_cb, g_object_ref (task));
 	}
 
 	if (data->n_pending_ops > 0)
 		return;
 
 	/* At this point, all the plugin->refine() calls are complete and the
-	 * gs_odrs_provider_refine_async() call is also complete. If an error
+	 * gs_odrs_provider_refine_async() and gs_rewrite_resources_async()
+	 * calls are also complete. If an error
 	 * occurred during those calls, return with it now rather than
 	 * proceeding to the recursive calls below. */
 	if (data->error != NULL) {
@@ -556,7 +659,13 @@ recursive_internal_refine_cb (GObject      *source_object,
 	g_autoptr(GTask) task = g_steal_pointer (&user_data);
 	g_autoptr(GError) local_error = NULL;
 
-	run_refine_internal_finish (self, result, &local_error);
+	if (!run_refine_internal_finish (self, result, &local_error) &&
+	    !g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+	    !g_error_matches (local_error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED)) {
+		g_debug ("failed to recursive-refine: %s", local_error->message);
+		g_clear_error (&local_error);
+	}
+
 	finish_refine_internal_recursion (task, g_steal_pointer (&local_error));
 }
 
@@ -645,6 +754,10 @@ gs_plugin_job_refine_run_async (GsPluginJob         *job,
 		g_object_freeze_notify (G_OBJECT (app));
 	}
 
+#ifdef HAVE_SYSPROF
+	self->begin_time_nsec = SYSPROF_CAPTURE_CURRENT_TIME;
+#endif
+
 	/* Start refining the apps. */
 	run_refine_internal_async (self, plugin_loader, result_list,
 				   self->flags, cancellable,
@@ -701,6 +814,7 @@ run_cb (GObject      *source_object,
 	/* Delayed error handling. */
 	if (local_error != NULL) {
 		gs_utils_error_convert_gio (&local_error);
+		g_signal_emit_by_name (G_OBJECT (self), "completed");
 		g_task_return_error (task, g_steal_pointer (&local_error));
 		return;
 	}
@@ -735,6 +849,7 @@ finish_run (GTask     *task,
 	/* success */
 	g_set_object (&self->result_list, result_list);
 	g_task_return_boolean (task, TRUE);
+	g_signal_emit_by_name (G_OBJECT (self), "completed");
 }
 
 static gboolean
@@ -742,6 +857,11 @@ gs_plugin_job_refine_run_finish (GsPluginJob   *self,
                                  GAsyncResult  *result,
                                  GError       **error)
 {
+	GS_PROFILER_ADD_MARK (PluginJobRefine,
+			      GS_PLUGIN_JOB_REFINE (self)->begin_time_nsec,
+			      G_OBJECT_TYPE_NAME (self),
+			      NULL);
+
 	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
