@@ -267,18 +267,6 @@ void		 gs_plugin_add_rule			(GsPlugin	*plugin,
 							 const gchar	*name);
 
 /* helpers */
-gboolean	 gs_plugin_download_file		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 const gchar	*uri,
-							 const gchar	*filename,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gchar		*gs_plugin_download_rewrite_resource	(GsPlugin	*plugin,
-							 GsApp		*app,
-							 const gchar	*resource,
-							 GCancellable	*cancellable,
-							 GError		**error);
-
 gboolean	 gs_plugin_check_distro_id		(GsPlugin	*plugin,
 							 const gchar	*distro_id);
 GsApp		*gs_plugin_cache_lookup			(GsPlugin	*plugin,
@@ -292,6 +280,7 @@ void		 gs_plugin_cache_add			(GsPlugin	*plugin,
 void		 gs_plugin_cache_remove			(GsPlugin	*plugin,
 							 const gchar	*key);
 void		 gs_plugin_cache_invalidate		(GsPlugin	*plugin);
+GsAppList	*gs_plugin_list_cached			(GsPlugin	*plugin);
 void		 gs_plugin_status_update		(GsPlugin	*plugin,
 							 GsApp		*app,
 							 GsPluginStatus	 status);
