@@ -4,7 +4,7 @@
  * Copyright (C) 2013-2016 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2018 Kalev Lember <klember@redhat.com>
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #pragma once
@@ -97,7 +97,6 @@ gboolean	 gs_utils_parse_evr		(const gchar	 *evr,
 						 gchar		**out_epoch,
 						 gchar		**out_version,
 						 gchar		**out_release);
-void		 gs_utils_set_online_updates_timestamp (GSettings *settings);
 
 gchar		*gs_utils_unique_id_compat_convert	(const gchar	*data_id);
 
@@ -142,4 +141,17 @@ gboolean	 gs_utils_set_file_etag		(GFile			*file,
 
 gchar		*gs_utils_get_upgrade_background (const gchar		*version);
 
+gint		 gs_utils_app_sort_name		(GsApp			*app1,
+						 GsApp			*app2,
+						 gpointer		 user_data);
+gint		 gs_utils_app_sort_match_value	(GsApp			*app1,
+						 GsApp			*app2,
+						 gpointer		 user_data);
+gint		 gs_utils_app_sort_priority	(GsApp			*app1,
+						 GsApp			*app2,
+						 gpointer		 user_data);
+void		 gs_utils_gstring_replace	(GString		*str,
+						 const gchar		*find,
+						 const gchar		*replace);
+
 G_END_DECLS
