@@ -39,10 +39,14 @@ gs_plugin_generic_updates_merge_os_update (GsApp *app)
 	    gs_app_get_scope (app) != AS_COMPONENT_SCOPE_SYSTEM)
 		return FALSE;
 
-	if (gs_app_get_kind (app) == AS_COMPONENT_KIND_GENERIC)
-		return TRUE;
-	if (gs_app_get_kind (app) == AS_COMPONENT_KIND_REPOSITORY)
+	switch (gs_app_get_kind (app)) {
+	case AS_COMPONENT_KIND_GENERIC:
+	case AS_COMPONENT_KIND_REPOSITORY:
+	case AS_COMPONENT_KIND_SERVICE:
 		return TRUE;
+	default:
+		break;
+	}
 
 	return FALSE;
 }
@@ -75,6 +79,7 @@ gs_plugin_generic_updates_get_os_update (GsPlugin *plugin)
 				gs_app_get_summary (app));
 	ic = g_themed_icon_new ("system-component-os-updates");
 	gs_app_add_icon (app, ic);
+
 	return app;
 }
 
@@ -89,12 +94,14 @@ gs_plugin_generic_updates_refine_async (GsPlugin            *plugin,
 	g_autoptr(GTask) task = NULL;
 	g_autoptr(GsApp) app = NULL;
 	g_autoptr(GsAppList) os_updates = gs_app_list_new ();
+	AsUrgencyKind max_urgency = AS_URGENCY_KIND_UNKNOWN;
 
 	task = g_task_new (plugin, cancellable, callback, user_data);
 	g_task_set_source_tag (task, gs_plugin_generic_updates_refine_async);
 
 	/* not from get_updates() */
-	if ((flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS) == 0) {
+	if ((flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS) == 0 &&
+	    (flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_SEVERITY) == 0) {
 		g_task_return_boolean (task, TRUE);
 		return;
 	}
@@ -104,8 +111,11 @@ gs_plugin_generic_updates_refine_async (GsPlugin            *plugin,
 		GsApp *app_tmp = gs_app_list_index (list, i);
 		if (gs_app_has_quirk (app_tmp, GS_APP_QUIRK_IS_WILDCARD))
 			continue;
-		if (gs_plugin_generic_updates_merge_os_update (app_tmp))
+		if (gs_plugin_generic_updates_merge_os_update (app_tmp)) {
+			if (max_urgency < gs_app_get_update_urgency (app_tmp))
+				max_urgency = gs_app_get_update_urgency (app_tmp);
 			gs_app_list_add (os_updates, app_tmp);
+		}
 	}
 	if (gs_app_list_length (os_updates) == 0) {
 		g_task_return_boolean (task, TRUE);
@@ -114,6 +124,7 @@ gs_plugin_generic_updates_refine_async (GsPlugin            *plugin,
 
 	/* create new meta object */
 	app = gs_plugin_generic_updates_get_os_update (plugin);
+	gs_app_set_update_urgency (app, max_urgency);
 	for (guint i = 0; i < gs_app_list_length (os_updates); i++) {
 		GsApp *app_tmp = gs_app_list_index (os_updates, i);
 		gs_app_add_related (app, app_tmp);
