@@ -42,14 +42,11 @@ gs_plugin_fedora_langpacks_init (GsPluginFedoraLangpacks *self)
 	* Example: en {en_GB}, pt {pt_BR}, zh {zh_CN, zh_TW, zh_HK}
 	*/
 	self->locale_langpack_map = g_hash_table_new (g_str_hash, g_str_equal);
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdiscarded-qualifiers"
-	g_hash_table_insert (self->locale_langpack_map, "en_GB", "langpacks-en_GB");
-	g_hash_table_insert (self->locale_langpack_map, "pt_BR", "langpacks-pt_BR");
-	g_hash_table_insert (self->locale_langpack_map, "zh_CN", "langpacks-zh_CN");
-	g_hash_table_insert (self->locale_langpack_map, "zh_TW", "langpacks-zh_TW");
-	g_hash_table_insert (self->locale_langpack_map, "zh_HK", "langpacks-zh_HK");
-#pragma GCC diagnostic pop
+	g_hash_table_insert (self->locale_langpack_map, (gpointer) "en_GB", (gpointer) "langpacks-en_GB");
+	g_hash_table_insert (self->locale_langpack_map, (gpointer) "pt_BR", (gpointer) "langpacks-pt_BR");
+	g_hash_table_insert (self->locale_langpack_map, (gpointer) "zh_CN", (gpointer) "langpacks-zh_CN");
+	g_hash_table_insert (self->locale_langpack_map, (gpointer) "zh_TW", (gpointer) "langpacks-zh_TW");
+	g_hash_table_insert (self->locale_langpack_map, (gpointer) "zh_HK", (gpointer) "langpacks-zh_HK");
 }
 
 static void
