@@ -393,7 +393,18 @@ _create_upgrade_from_info (GsPluginFedoraPkgdbCollections *self,
 
 	/* create */
 	app = gs_app_new (app_id);
-	gs_app_set_state (app, GS_APP_STATE_AVAILABLE);
+	switch (item->status) {
+	case PKGDB_ITEM_STATUS_ACTIVE:
+	case PKGDB_ITEM_STATUS_DEVEL:
+		gs_app_set_state (app, GS_APP_STATE_UPDATABLE);
+		break;
+	case PKGDB_ITEM_STATUS_EOL:
+		gs_app_set_state (app, GS_APP_STATE_UNAVAILABLE);
+		break;
+	default:
+		gs_app_set_state (app, GS_APP_STATE_AVAILABLE);
+		break;
+	}
 	gs_app_set_kind (app, AS_COMPONENT_KIND_OPERATING_SYSTEM);
 	gs_app_set_bundle_kind (app, AS_BUNDLE_KIND_PACKAGE);
 	gs_app_set_name (app, GS_APP_QUALITY_LOWEST, item->name);
@@ -410,7 +421,7 @@ _create_upgrade_from_info (GsPluginFedoraPkgdbCollections *self,
 	gs_app_add_icon (app, ic);
 
 	/* show a Fedora magazine article for the release */
-	url = g_strdup_printf ("https://fedoramagazine.org/whats-new-fedora-%u-workstation",
+	url = g_strdup_printf ("https://fedoramagazine.org/whats-new-fedora-workstation-%u",
 			       item->version);
 	gs_app_set_url (app, AS_URL_KIND_HOMEPAGE, url);
 
@@ -447,6 +458,10 @@ _is_valid_upgrade (GsPluginFedoraPkgdbCollections *self,
 	    item->version > self->os_version + 2)
 		return FALSE;
 
+	/* ignore End-Of-Life upgrades */
+	if (item->status == PKGDB_ITEM_STATUS_EOL)
+		return FALSE;
+
 	/* only interested in non-devel distros */
 	if (!g_settings_get_boolean (self->settings, "show-upgrade-prerelease")) {
 		if (item->status == PKGDB_ITEM_STATUS_DEVEL)
@@ -697,7 +712,7 @@ refine_app (GsPluginFedoraPkgdbCollections *self,
 		app_version = g_ascii_strtoull (gs_app_get_version (app), NULL, 10);
 
 	/* system updates and system upgrades are the same kind, only different version */
-	if (app_version != self->os_version)
+	if (app_version == 0 || app_version == self->os_version)
 		return TRUE;
 
 	/* find item */
@@ -744,7 +759,6 @@ gs_plugin_fedora_pkgdb_collections_refine_async (GsPlugin            *plugin,
 	GsPluginFedoraPkgdbCollections *self = GS_PLUGIN_FEDORA_PKGDB_COLLECTIONS (plugin);
 	g_autoptr(GTask) task = NULL;
 	gboolean refine_needed = FALSE;
-	g_autoptr(GError) local_error = NULL;
 
 	task = gs_plugin_refine_data_new_task (plugin, list, flags, cancellable, callback, user_data);
 	g_task_set_source_tag (task, gs_plugin_fedora_pkgdb_collections_refine_async);
