@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2018 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #pragma once
@@ -13,11 +13,17 @@
 
 G_BEGIN_DECLS
 
+typedef enum {
+	GS_FLATPAK_ERROR_MODE_IGNORE_ERRORS = 0,
+	GS_FLATPAK_ERROR_MODE_STOP_ON_FIRST_ERROR = 1,
+} GsFlatpakErrorMode;
+
 #define GS_TYPE_FLATPAK_TRANSACTION (gs_flatpak_transaction_get_type ())
 
 G_DECLARE_FINAL_TYPE (GsFlatpakTransaction, gs_flatpak_transaction, GS, FLATPAK_TRANSACTION, FlatpakTransaction)
 
 FlatpakTransaction	*gs_flatpak_transaction_new		(FlatpakInstallation	*installation,
+								 gboolean		 stop_on_first_error,
 								 GCancellable		*cancellable,
 								 GError			**error);
 GsApp			*gs_flatpak_transaction_get_app_by_ref	(FlatpakTransaction	*transaction,
@@ -27,9 +33,5 @@ void			 gs_flatpak_transaction_add_app		(FlatpakTransaction	*transaction,
 gboolean		 gs_flatpak_transaction_run		(FlatpakTransaction	*transaction,
 								 GCancellable		*cancellable,
 								 GError			**error);
-#if !FLATPAK_CHECK_VERSION(1,5,1)
-void			 gs_flatpak_transaction_set_no_deploy	(FlatpakTransaction	*transaction,
-								 gboolean		 no_deploy);
-#endif
 
 G_END_DECLS
