@@ -102,7 +102,7 @@ gs_plugins_flatpak_repo_non_ascii_func (GsPluginLoader *plugin_loader)
 	gs_test_flush_main_context ();
 	g_assert_no_error (error);
 	g_assert_true (app != NULL);
-	g_assert_cmpstr (gs_app_get_unique_id (app), ==, "*/*/*/example__1____/master");
+	g_assert_cmpstr (gs_app_get_unique_id (app), ==, "user/*/*/example__1____/master");
 }
 
 static void
@@ -162,7 +162,7 @@ gs_plugins_flatpak_repo_func (GsPluginLoader *plugin_loader)
 			 "Longer multiline comment that does into detail.");
 	g_assert_true (gs_app_get_local_file (app) != NULL);
 	/* The app has an icon, but cannot be found since it is not installed */
-	g_assert_nonnull (gs_app_get_icons (app));
+	g_assert_true (gs_app_has_icons (app));
 	icon = gs_app_get_icon_for_size (app, 64, 1, NULL);
 	g_assert_null (icon);
 
@@ -870,7 +870,7 @@ gs_plugins_flatpak_runtime_repo_redundant_func (GsPluginLoader *plugin_loader)
 	g_assert_cmpint (gs_app_get_kind (app_src), ==, AS_COMPONENT_KIND_REPOSITORY);
 	g_assert_cmpint (gs_app_get_state (app_src), ==, GS_APP_STATE_AVAILABLE_LOCAL);
 	g_assert_cmpstr (gs_app_get_id (app_src), ==, "test");
-	g_assert_cmpstr (gs_app_get_unique_id (app_src), ==, "*/*/*/test/master");
+	g_assert_cmpstr (gs_app_get_unique_id (app_src), ==, "user/*/*/test/master");
 	g_assert_true (gs_app_get_local_file (app_src) != NULL);
 
 	/* install the source manually */
@@ -1630,6 +1630,9 @@ gs_plugins_flatpak_app_update_func (GsPluginLoader *plugin_loader)
 	g_assert_no_error (error);
 	g_assert_true (ret);
 	g_assert_cmpint (gs_app_get_state (app_source), ==, GS_APP_STATE_UNAVAILABLE);
+
+	/* to not have deleted the "flatpak/tests/app-update/repo" content */
+	unlink (repo_path);
 }
 
 static void
