@@ -4,7 +4,7 @@
  * Copyright (C) 2012 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2014-2015 Kalev Lember <klember@redhat.com>
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #pragma once
@@ -21,9 +21,7 @@ G_DECLARE_FINAL_TYPE (GsAppAddonRow, gs_app_addon_row, GS, APP_ADDON_ROW, GtkLis
 
 GtkWidget	*gs_app_addon_row_new			(GsApp		*app);
 void		 gs_app_addon_row_refresh		(GsAppAddonRow	*row);
-void		 gs_app_addon_row_set_selected		(GsAppAddonRow	*row,
-							 gboolean        selected);
-gboolean	 gs_app_addon_row_get_selected		(GsAppAddonRow	*row);
+void		 gs_app_addon_row_activate		(GsAppAddonRow	*row);
 GsApp		*gs_app_addon_row_get_addon		(GsAppAddonRow	*row);
 
 G_END_DECLS
