@@ -20,7 +20,6 @@
 
 #include "config.h"
 
-#include <adwaita.h>
 #include <glib/gi18n.h>
 
 #include "gs-app-details-page.h"
@@ -31,39 +30,27 @@
 typedef enum {
 	PROP_APP = 1,
 	PROP_PLUGIN_LOADER,
-	PROP_SHOW_BACK_BUTTON,
-	PROP_TITLE,
 } GsAppDetailsPageProperty;
 
-enum {
-	SIGNAL_BACK_CLICKED,
-	SIGNAL_LAST
-};
-
-static GParamSpec *obj_props[PROP_TITLE + 1] = { NULL, };
-
-static guint signals[SIGNAL_LAST] = { 0 };
+static GParamSpec *obj_props[PROP_PLUGIN_LOADER + 1] = { NULL, };
 
 struct _GsAppDetailsPage
 {
 	GtkBox		 parent_instance;
 
-	GtkWidget	*back_button;
-	GtkWidget	*header_bar;
 	GtkWidget	*label_details;
 	GtkWidget	*spinner_details;
 	GtkWidget	*permissions_section;
 	GtkWidget	*permissions_section_list;
 	GtkWidget	*status_page;
 	GtkWidget	*status_page_clamp;
-	AdwWindowTitle	*window_title;
 
 	GsPluginLoader	*plugin_loader; /* (owned) */
 	GsApp		*app;  /* (owned) (nullable) */
 	GCancellable	*refine_cancellable; /* (owned) (nullable) */
 };
 
-G_DEFINE_TYPE (GsAppDetailsPage, gs_app_details_page, GTK_TYPE_BOX)
+G_DEFINE_TYPE (GsAppDetailsPage, gs_app_details_page, ADW_TYPE_NAVIGATION_PAGE)
 
 static const struct {
         GsAppPermissionsFlags permission;
@@ -74,7 +61,10 @@ static const struct {
   { GS_APP_PERMISSIONS_FLAGS_SYSTEM_BUS, N_("System Services"), N_("Can access D-Bus services on the system bus") },
   { GS_APP_PERMISSIONS_FLAGS_SESSION_BUS, N_("Session Services"), N_("Can access D-Bus services on the session bus") },
   { GS_APP_PERMISSIONS_FLAGS_DEVICES, N_("Devices"), N_("Can access arbitrary devices such as webcams") },
+  { GS_APP_PERMISSIONS_FLAGS_INPUT_DEVICES, N_("Devices"), N_("Can access input devices") },
+  { GS_APP_PERMISSIONS_FLAGS_PULSEAUDIO_DEVICES, N_("Devices"), N_("Can access microphones") },
   { GS_APP_PERMISSIONS_FLAGS_SYSTEM_DEVICES, N_("Devices"), N_("Can access system device files") },
+  { GS_APP_PERMISSIONS_FLAGS_SCREEN, N_("Screen contents"), N_("Can access screen contents") },
   { GS_APP_PERMISSIONS_FLAGS_HOME_FULL, N_("Home folder"), N_("Can view, edit and create files") },
   { GS_APP_PERMISSIONS_FLAGS_HOME_READ, N_("Home folder"), N_("Can view files") },
   { GS_APP_PERMISSIONS_FLAGS_FILESYSTEM_FULL, N_("File system"), N_("Can view, edit and create files") },
@@ -245,9 +235,8 @@ set_updates_description_ui (GsAppDetailsPage *page, GsApp *app)
 
 	/* FIXME support app == NULL */
 
-	/* set window title */
-	adw_window_title_set_title (page->window_title, _("Update Details"));
-	g_object_notify_by_pspec (G_OBJECT (page), obj_props[PROP_TITLE]);
+	/* set page title */
+	adw_navigation_page_set_title (ADW_NAVIGATION_PAGE (page), _("Update Details"));
 
 	/* set update header */
 	set_update_description (page, TRUE);
@@ -337,53 +326,6 @@ gs_app_details_page_set_app (GsAppDetailsPage *page, GsApp *app)
 	g_object_notify_by_pspec (G_OBJECT (page), obj_props[PROP_APP]);
 }
 
-/**
- * gs_app_details_page_get_show_back_button:
- * @page: a #GsAppDetailsPage
- *
- * Get the value of #GsAppDetailsPage:show-back-button.
- *
- * Returns: whether to show the back button
- *
- * Since: 41
- */
-gboolean
-gs_app_details_page_get_show_back_button (GsAppDetailsPage *page)
-{
-	g_return_val_if_fail (GS_IS_APP_DETAILS_PAGE (page), FALSE);
-	return gtk_widget_get_visible (page->back_button);
-}
-
-/**
- * gs_app_details_page_set_show_back_button:
- * @page: a #GsAppDetailsPage
- * @show_back_button: whether to show the back button
- *
- * Set the value of #GsAppDetailsPage:show-back-button.
- *
- * Since: 41
- */
-void
-gs_app_details_page_set_show_back_button (GsAppDetailsPage *page, gboolean show_back_button)
-{
-	g_return_if_fail (GS_IS_APP_DETAILS_PAGE (page));
-
-	show_back_button = !!show_back_button;
-
-	if (gtk_widget_get_visible (page->back_button) == show_back_button)
-		return;
-
-	gtk_widget_set_visible (page->back_button, show_back_button);
-
-	g_object_notify_by_pspec (G_OBJECT (page), obj_props[PROP_SHOW_BACK_BUTTON]);
-}
-
-static void
-back_clicked_cb (GtkWidget *widget, GsAppDetailsPage *page)
-{
-	g_signal_emit (page, signals[SIGNAL_BACK_CLICKED], 0);
-}
-
 static void
 gs_app_details_page_dispose (GObject *object)
 {
@@ -411,12 +353,6 @@ gs_app_details_page_get_property (GObject *object, guint prop_id, GValue *value,
 	case PROP_PLUGIN_LOADER:
 		g_value_set_object (value, gs_app_details_page_get_plugin_loader (page));
 		break;
-	case PROP_SHOW_BACK_BUTTON:
-		g_value_set_boolean (value, gs_app_details_page_get_show_back_button (page));
-		break;
-	case PROP_TITLE:
-		g_value_set_string (value, adw_window_title_get_title (page->window_title));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -436,9 +372,6 @@ gs_app_details_page_set_property (GObject *object, guint prop_id, const GValue *
 		g_assert (page->plugin_loader == NULL);
 		page->plugin_loader = g_value_dup_object (value);
 		break;
-	case PROP_SHOW_BACK_BUTTON:
-		gs_app_details_page_set_show_back_button (page, g_value_get_boolean (value));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -504,59 +437,16 @@ gs_app_details_page_class_init (GsAppDetailsPageClass *klass)
 				     GS_TYPE_PLUGIN_LOADER,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
-	/**
-	 * GsAppDetailsPage:show-back-button
-	 *
-	 * Whether to show the back button.
-	 *
-	 * Since: 41
-	 */
-	obj_props[PROP_SHOW_BACK_BUTTON] =
-		g_param_spec_boolean ("show-back-button", NULL, NULL,
-				     TRUE,
-				     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
-	/**
-	 * GsAppDetailsPage:title
-	 *
-	 * Read-only window title.
-	 *
-	 * Since: 42
-	 */
-	obj_props[PROP_TITLE] =
-		g_param_spec_string ("title", NULL, NULL,
-				     NULL,
-				     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
 	g_object_class_install_properties (object_class, G_N_ELEMENTS (obj_props), obj_props);
 
-	/**
-	 * GsAppDetailsPage:back-clicked:
-	 * @app: a #GsApp
-	 *
-	 * Emitted when the back button got activated and the #GsUpdateDialog
-	 * containing this page is expected to go back.
-	 *
-	 * Since: 41
-	 */
-	signals[SIGNAL_BACK_CLICKED] =
-		g_signal_new ("back-clicked",
-			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
-			      0, NULL, NULL, g_cclosure_marshal_generic,
-			      G_TYPE_NONE, 0);
-
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Software/gs-app-details-page.ui");
 
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, back_button);
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, header_bar);
 	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, label_details);
 	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, spinner_details);
 	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, permissions_section);
 	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, permissions_section_list);
 	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, status_page);
 	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, status_page_clamp);
-	gtk_widget_class_bind_template_child (widget_class, GsAppDetailsPage, window_title);
-	gtk_widget_class_bind_template_callback (widget_class, back_clicked_cb);
 }
 
 /**
