@@ -8,7 +8,7 @@
 
 #pragma once
 
-#include <gtk/gtk.h>
+#include <adwaita.h>
 
 #include "gnome-software-private.h"
 
@@ -16,7 +16,7 @@ G_BEGIN_DECLS
 
 #define GS_TYPE_APP_REVIEWS_DIALOG (gs_app_reviews_dialog_get_type ())
 
-G_DECLARE_FINAL_TYPE (GsAppReviewsDialog, gs_app_reviews_dialog, GS, APP_REVIEWS_DIALOG, GtkDialog)
+G_DECLARE_FINAL_TYPE (GsAppReviewsDialog, gs_app_reviews_dialog, GS, APP_REVIEWS_DIALOG, AdwWindow)
 
 GtkWidget	*gs_app_reviews_dialog_new	(GtkWindow	*parent,
 						 GsApp		*app,
