@@ -39,6 +39,7 @@ typedef struct
 	GtkWidget	*box_tag;
 	GtkWidget	*label_warning;
 	GtkWidget	*label_origin;
+	GtkWidget	*label_installed_box;
 	GtkWidget	*label_installed;
 	GtkWidget	*label_app_size;
 	gboolean	 colorful;
@@ -180,9 +181,15 @@ gs_app_row_refresh_button (GsAppRow *app_row, gboolean missing_search_result)
 	case GS_APP_STATE_UPDATABLE_LIVE:
 		gtk_widget_set_visible (priv->button, TRUE);
 		if (priv->show_update) {
-			/* TRANSLATORS: this is a button in the updates panel
-			 * that allows the app to be easily updated live */
-			gs_progress_button_set_label (GS_PROGRESS_BUTTON (priv->button), _("Update"));
+			if (gs_app_has_quirk (priv->app, GS_APP_QUIRK_NEEDS_REBOOT) &&
+			    !gs_app_is_downloaded (priv->app)) {
+				/* TRANSLATORS: this is a button in the updates panel */
+				gs_progress_button_set_label (GS_PROGRESS_BUTTON (priv->button), _("Download"));
+			} else {
+				/* TRANSLATORS: this is a button in the updates panel
+				 * that allows the app to be easily updated live */
+				gs_progress_button_set_label (GS_PROGRESS_BUTTON (priv->button), _("Update"));
+			}
 			gs_progress_button_set_icon_name (GS_PROGRESS_BUTTON (priv->button), "software-update-available-symbolic");
 			gtk_widget_set_sensitive (priv->button, !gs_app_has_quirk (priv->app, GS_APP_QUIRK_NEEDS_USER_ACTION));
 		} else {
@@ -215,6 +222,13 @@ gs_app_row_refresh_button (GsAppRow *app_row, gboolean missing_search_result)
 		gs_progress_button_set_label (GS_PROGRESS_BUTTON (priv->button), _("Uninstalling"));
 		gs_progress_button_set_icon_name (GS_PROGRESS_BUTTON (priv->button), NULL);
 		break;
+	case GS_APP_STATE_DOWNLOADING:
+		gtk_widget_set_visible (priv->button, TRUE);
+		/* TRANSLATORS: this is a button next to the search results that
+		 * shows the status of an app being downloaded */
+		gs_progress_button_set_label (GS_PROGRESS_BUTTON (priv->button), _("Downloading"));
+		gs_progress_button_set_icon_name (GS_PROGRESS_BUTTON (priv->button), NULL);
+		break;
 	default:
 		break;
 	}
@@ -228,6 +242,7 @@ gs_app_row_refresh_button (GsAppRow *app_row, gboolean missing_search_result)
 	case GS_APP_STATE_UPDATABLE_LIVE:
 	case GS_APP_STATE_INSTALLING:
 	case GS_APP_STATE_REMOVING:
+	case GS_APP_STATE_DOWNLOADING:
 		gtk_widget_set_visible (priv->button, TRUE);
 		break;
 	case GS_APP_STATE_UPDATABLE:
@@ -266,6 +281,7 @@ gs_app_row_refresh_button (GsAppRow *app_row, gboolean missing_search_result)
 	switch (gs_app_get_state (priv->app)) {
 	case GS_APP_STATE_INSTALLING:
 	case GS_APP_STATE_REMOVING:
+	case GS_APP_STATE_DOWNLOADING:
 		gtk_widget_set_sensitive (priv->button, FALSE);
 		break;
 	default:
@@ -276,6 +292,62 @@ gs_app_row_refresh_button (GsAppRow *app_row, gboolean missing_search_result)
 	gs_app_row_update_button_reveal (app_row);
 }
 
+static void
+append_to_name_if_meaningful (GPtrArray *name_parts, GtkLabel *label)
+{
+	const gchar *text = gtk_label_get_text (label);
+	if (gtk_widget_is_visible (GTK_WIDGET (label)) && text != NULL && text[0] != '\0') {
+		g_ptr_array_add (name_parts, (gpointer) text);
+	}
+}
+
+static void
+gs_app_row_update_accessible_name (GsAppRow *app_row)
+{
+	GsAppRowPrivate *priv = gs_app_row_get_instance_private (app_row);
+	/* typically not more than 3-4 of these widgets are actually visible, hence 5 below */
+	g_autoptr(GPtrArray) parts = g_ptr_array_new_null_terminated (5, NULL, TRUE);
+	g_autofree char *accessible_name = NULL;
+	gboolean is_rtl = gtk_widget_get_direction (GTK_WIDGET (app_row)) == GTK_TEXT_DIR_RTL;
+
+	/* As this is a complex widget, the screen reader doesn’t read it all
+	 * out correctly by default, so we provide an override label. The label
+	 * contains the textual versions of most of the widgets in the row, in
+	 * the order they appear visually. This order differs in RTL
+	 * environments, where each sub-row of the app row is reversed. In
+	 * practice, that means only the name/critical and the version. */
+
+	g_ptr_array_add (parts, (gpointer) gtk_label_get_text (GTK_LABEL (priv->name_label)));
+
+	if (gtk_widget_get_visible (priv->update_critical_image))
+		g_ptr_array_insert (parts, is_rtl ? 0 : -1, _("Critical update"));
+
+	append_to_name_if_meaningful (parts, GTK_LABEL (priv->description_label));
+
+	if (!is_rtl) {
+		append_to_name_if_meaningful (parts, GTK_LABEL (priv->version_current_label));
+		append_to_name_if_meaningful (parts, GTK_LABEL (priv->version_arrow_label));
+		append_to_name_if_meaningful (parts, GTK_LABEL (priv->version_update_label));
+	} else {
+		append_to_name_if_meaningful (parts, GTK_LABEL (priv->version_update_label));
+		append_to_name_if_meaningful (parts, GTK_LABEL (priv->version_arrow_label));
+		append_to_name_if_meaningful (parts, GTK_LABEL (priv->version_current_label));
+	}
+
+	/* each of these are visually on a separate row, so don’t need RTL treatment */
+	append_to_name_if_meaningful (parts, GTK_LABEL (priv->label_installed));
+	append_to_name_if_meaningful (parts, GTK_LABEL (priv->label_app_size));
+	append_to_name_if_meaningful (parts, GTK_LABEL (priv->label_origin));
+	append_to_name_if_meaningful (parts, GTK_LABEL (priv->system_updates_label));
+	append_to_name_if_meaningful (parts, GTK_LABEL (priv->label_warning));
+	append_to_name_if_meaningful (parts, GTK_LABEL (priv->label));
+
+	accessible_name = g_strjoinv (" ", (gchar **) parts->pdata);
+	gtk_accessible_update_property (GTK_ACCESSIBLE (app_row),
+					GTK_ACCESSIBLE_PROPERTY_LABEL, accessible_name,
+					-1);
+}
+
 static void
 gs_app_row_actually_refresh (GsAppRow *app_row)
 {
@@ -298,6 +370,7 @@ gs_app_row_actually_refresh (GsAppRow *app_row)
 	/* do a fill bar for the current progress */
 	switch (gs_app_get_state (priv->app)) {
 	case GS_APP_STATE_INSTALLING:
+	case GS_APP_STATE_DOWNLOADING:
 		gs_progress_button_set_progress (GS_PROGRESS_BUTTON (priv->button),
 		                                 gs_app_get_progress (priv->app));
 		gs_progress_button_set_show_progress (GS_PROGRESS_BUTTON (priv->button), TRUE);
@@ -349,14 +422,14 @@ gs_app_row_actually_refresh (GsAppRow *app_row)
 		case GS_APP_STATE_UPDATABLE:
 		case GS_APP_STATE_UPDATABLE_LIVE:
 		case GS_APP_STATE_INSTALLED:
-			gtk_widget_set_visible (priv->label_installed, priv->show_installed);
+			gtk_widget_set_visible (priv->label_installed_box, priv->show_installed);
 			break;
 		default:
-			gtk_widget_set_visible (priv->label_installed, FALSE);
+			gtk_widget_set_visible (priv->label_installed_box, FALSE);
 			break;
 		}
 	} else {
-		gtk_widget_set_visible (priv->label_installed, FALSE);
+		gtk_widget_set_visible (priv->label_installed_box, FALSE);
 	}
 
 	/* name */
@@ -489,6 +562,7 @@ gs_app_row_actually_refresh (GsAppRow *app_row)
 					!gs_app_has_quirk (priv->app, GS_APP_QUIRK_NEEDS_USER_ACTION));
 		break;
 	case GS_APP_STATE_INSTALLING:
+	case GS_APP_STATE_DOWNLOADING:
 		gtk_widget_set_visible (priv->button_box, TRUE);
 		break;
 	default:
@@ -576,11 +650,13 @@ gs_app_row_actually_refresh (GsAppRow *app_row)
 
 	gtk_widget_set_visible (priv->box_tag,
 				gtk_widget_get_visible (priv->label_origin) ||
-				gtk_widget_get_visible (priv->label_installed) ||
+				gtk_widget_get_visible (priv->label_installed_box) ||
 				gtk_widget_get_visible (priv->label_warning));
 
 	gtk_label_set_max_width_chars (GTK_LABEL (priv->name_label),
 				       gtk_widget_get_visible (priv->description_label) ? 20 : -1);
+
+	gs_app_row_update_accessible_name (app_row);
 }
 
 static void
@@ -1004,6 +1080,7 @@ gs_app_row_class_init (GsAppRowClass *klass)
 	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, box_tag);
 	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label_warning);
 	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label_origin);
+	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label_installed_box);
 	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label_installed);
 	gtk_widget_class_bind_template_child_private (widget_class, GsAppRow, label_app_size);
 }
