@@ -17,12 +17,12 @@
 
 struct _GsAppVersionHistoryDialog
 {
-	GtkDialog	 parent_instance;
+	AdwWindow	 parent_instance;
 	GsApp		*app;
 	GtkWidget	*listbox;
 };
 
-G_DEFINE_TYPE (GsAppVersionHistoryDialog, gs_app_version_history_dialog, GTK_TYPE_DIALOG)
+G_DEFINE_TYPE (GsAppVersionHistoryDialog, gs_app_version_history_dialog, ADW_TYPE_WINDOW)
 
 static void
 populate_version_history (GsAppVersionHistoryDialog *dialog,
@@ -33,6 +33,9 @@ populate_version_history (GsAppVersionHistoryDialog *dialog,
 	/* remove previous */
 	gs_widget_remove_all (dialog->listbox, (GsRemoveFunc) gtk_list_box_remove);
 
+	/* grab focus so first row text is not selected by default */
+	gtk_widget_grab_focus (dialog->listbox);
+
 	version_history = gs_app_get_version_history (app);
 	if (version_history == NULL || version_history->len == 0) {
 		GtkWidget *row;
@@ -77,6 +80,8 @@ gs_app_version_history_dialog_class_init (GsAppVersionHistoryDialogClass *klass)
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Software/gs-app-version-history-dialog.ui");
 
 	gtk_widget_class_bind_template_child (widget_class, GsAppVersionHistoryDialog, listbox);
+
+	gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "window.close", NULL);
 }
 
 GtkWidget *
@@ -85,7 +90,6 @@ gs_app_version_history_dialog_new (GtkWindow *parent, GsApp *app)
 	GsAppVersionHistoryDialog *dialog;
 
 	dialog = g_object_new (GS_TYPE_APP_VERSION_HISTORY_DIALOG,
-			       "use-header-bar", TRUE,
 			       "transient-for", parent,
 			       "modal", TRUE,
 			       NULL);
