@@ -102,6 +102,8 @@ gs_app_version_history_row_class_init (GsAppVersionHistoryRowClass *klass)
 static void
 gs_app_version_history_row_init (GsAppVersionHistoryRow *row)
 {
+	g_type_ensure (GS_TYPE_DESCRIPTION_BOX);
+
 	gtk_widget_init_template (GTK_WIDGET (row));
 }
 
@@ -160,8 +162,10 @@ gs_app_version_history_row_set_info (GsAppVersionHistoryRow *row,
 
 	if (version_date_string == NULL)
 		gtk_widget_set_visible (row->version_date_label, FALSE);
-	else
+	else {
 		gtk_label_set_label (GTK_LABEL (row->version_date_label), version_date_string);
+		gtk_widget_set_visible (row->version_date_label, TRUE);
+	}
 
 	if (version_date_string_tooltip != NULL)
 		gtk_widget_set_tooltip_text (row->version_date_label, version_date_string_tooltip);
