@@ -468,7 +468,11 @@ gs_utils_widget_set_css (GtkWidget *widget, GtkCssProvider **provider, const gch
 	}
 
 	/* set up custom provider and store on the widget */
+	#if GTK_CHECK_VERSION(4, 12, 0)
+	gtk_css_provider_load_from_string (*provider, str->str);
+	#else
 	gtk_css_provider_load_from_data (*provider, str->str, -1);
+	#endif
 	gtk_style_context_add_provider_for_display (display, GTK_STYLE_PROVIDER (*provider),
 						    GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 }
@@ -601,14 +605,18 @@ gs_utils_show_error_dialog_simple (GtkWindow *parent,
 				   const gchar *text)
 {
 #ifndef TESTDATADIR
-	GtkWidget *window, *button, *container, *text_view, *vbox;
+	GtkWidget *window, *button, *container, *text_view, *vbox, *hbox, *label;
+	g_autoptr(PangoAttrList) bold = NULL;
 
 	g_return_if_fail (text != NULL);
 
+	bold = pango_attr_list_new ();
+	pango_attr_list_insert (bold, pango_attr_weight_new (PANGO_WEIGHT_BOLD));
+
 	window = adw_window_new ();
 	g_object_set (window,
 		      "modal", TRUE,
-		      "title", title,
+		      "title", " ",
 		      "destroy-with-parent", TRUE,
 		      "default-width", 500,
 		      "default-height", 350,
@@ -618,20 +626,62 @@ gs_utils_show_error_dialog_simple (GtkWindow *parent,
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	adw_window_set_content (ADW_WINDOW (window), vbox);
 
-	container = gtk_header_bar_new ();
+	container = adw_header_bar_new ();
+	gtk_widget_add_css_class (container, "flat");
 	gtk_box_append (GTK_BOX (vbox), container);
 
+	label = gtk_label_new (title);
+	g_object_set (label,
+		      "halign", GTK_ALIGN_CENTER,
+		      "margin-start", 12,
+		      "margin-end", 12,
+		      "margin-top", 0,
+		      "margin-bottom", 12,
+		      "justify", GTK_JUSTIFY_CENTER,
+		      "wrap-mode", PANGO_WRAP_WORD_CHAR,
+		      "wrap", TRUE,
+		      "width-chars", 40,
+		      "max-width-chars", 40,
+		      NULL);
+	gtk_widget_add_css_class (label, "title-4");
+	gtk_box_append (GTK_BOX (vbox), label);
+
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	g_object_set (hbox,
+		      "halign", GTK_ALIGN_FILL,
+		      "hexpand", TRUE,
+		      "margin-start", 12,
+		      "margin-end", 12,
+		      "margin-bottom", 6,
+		      NULL);
+	gtk_box_append (GTK_BOX (vbox), hbox);
+
+	label = gtk_label_new (_("Details"));
+	g_object_set (label,
+		      "halign", GTK_ALIGN_START,
+		      "hexpand", TRUE,
+		      "attributes", bold,
+		      NULL);
+	gtk_box_append (GTK_BOX (hbox), label);
+
 	button = gtk_button_new_from_icon_name ("edit-copy-symbolic");
 	g_object_set (button,
 		      "focus-on-click", FALSE,
 		      NULL);
-	gtk_header_bar_pack_start (GTK_HEADER_BAR (container), button);
+	gtk_widget_set_halign (button, GTK_ALIGN_END);
+	gtk_widget_add_css_class (button, "flat");
+	gtk_box_append (GTK_BOX (hbox), button);
 
 	container = gtk_scrolled_window_new ();
 	g_object_set (container,
 		      "hscrollbar-policy", GTK_POLICY_AUTOMATIC,
 		      "vscrollbar-policy", GTK_POLICY_AUTOMATIC,
+		      "margin-start", 12,
+		      "margin-end", 12,
+		      "margin-bottom", 12,
+		      "has-frame", TRUE,
 		      NULL);
+	gtk_widget_add_css_class (container, "gs-rounded");
 	gtk_box_append (GTK_BOX (vbox), container);
 
 	text_view = gtk_text_view_new ();
@@ -646,6 +696,7 @@ gs_utils_show_error_dialog_simple (GtkWindow *parent,
 		      "top-margin", 12,
 		      "bottom-margin", 12,
 		      NULL);
+	gtk_widget_add_css_class (text_view, "gs-rounded");
 	gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view)), text, -1);
 	gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (container), text_view);
 
@@ -1062,7 +1113,6 @@ gs_utils_invoke_reboot_ready2_cb (GObject *source_object,
 	if (ret_val != NULL) {
 		g_task_return_boolean (task, TRUE);
 	} else {
-		g_autoptr(GDBusConnection) bus = NULL;
 		GCancellable *cancellable;
 		const gchar *method_name = g_task_get_task_data (task);
 
