@@ -461,6 +461,7 @@ gs_hardware_support_context_dialog_get_display_support (GdkMonitor     *monitor,
 			AsRelationCompare comparator = as_relation_get_compare (relation);
 			Range current_display_comparand, relation_comparand;
 
+#if !AS_CHECK_VERSION(1, 0, 0)
 			/* From https://www.freedesktop.org/software/appstream/docs/chap-Metadata.html#tag-requires-recommends-display_length */
 			Range display_lengths[] = {
 				[AS_DISPLAY_LENGTH_KIND_XSMALL] = { 0, 360 },
@@ -469,6 +470,7 @@ gs_hardware_support_context_dialog_get_display_support (GdkMonitor     *monitor,
 				[AS_DISPLAY_LENGTH_KIND_LARGE] = { 1024, 3840 },
 				[AS_DISPLAY_LENGTH_KIND_XLARGE] = { 3840, G_MAXUINT },
 			};
+#endif
 
 			any_display_relations_set = TRUE;
 
@@ -485,11 +487,14 @@ gs_hardware_support_context_dialog_get_display_support (GdkMonitor     *monitor,
 			case AS_DISPLAY_SIDE_KIND_LAST:
 			default:
 				current_display_comparand.min = current_display_comparand.max = MAX (current_screen_size.width, current_screen_size.height);
+#if !AS_CHECK_VERSION(1, 0, 0)
 				relation_comparand.min = display_lengths[as_relation_get_value_display_length_kind (relation)].min;
 				relation_comparand.max = display_lengths[as_relation_get_value_display_length_kind (relation)].max;
+#endif
 				break;
 			}
 
+#if !AS_CHECK_VERSION(1, 0, 0)
 			if (evaluate_display_comparison (display_lengths[AS_DISPLAY_LENGTH_KIND_SMALL], comparator, relation_comparand)) {
 				*mobile_relation_kind_out = max_relation_kind (*mobile_relation_kind_out, as_relation_get_kind (relation));
 				*mobile_match_out = TRUE;
@@ -499,6 +504,7 @@ gs_hardware_support_context_dialog_get_display_support (GdkMonitor     *monitor,
 				*desktop_relation_kind_out = max_relation_kind (*desktop_relation_kind_out, as_relation_get_kind (relation));
 				*desktop_match_out = TRUE;
 			}
+#endif
 
 			if (evaluate_display_comparison (current_display_comparand, comparator, relation_comparand)) {
 				*current_relation_kind_out = max_relation_kind (*current_relation_kind_out, as_relation_get_kind (relation));
@@ -718,12 +724,19 @@ update_relations_list (GsHardwareSupportContextDialog *self)
 		title = g_strdup_printf (_("%s works on this device"), gs_app_get_name (self->app));
 		css_class = "green";
 		break;
+	case GS_CONTEXT_DIALOG_ROW_IMPORTANCE_INFORMATION:
+		icon_name = "app-installed-symbolic";
+		/* Translators: The app will possbily work on the current hardware.
+		 * The placeholder is the app name. */
+		title = g_strdup_printf (_("%s possibly works on this device"), gs_app_get_name (self->app));
+		css_class = "yellow";
+		break;
 	case GS_CONTEXT_DIALOG_ROW_IMPORTANCE_WARNING:
 		icon_name = "dialog-question-symbolic";
 		/* Translators: The app may not work fully on the current hardware.
 		 * The placeholder is the app name. */
 		title = g_strdup_printf (_("%s will not work properly on this device"), gs_app_get_name (self->app));
-		css_class = "yellow";
+		css_class = "orange";
 		break;
 	case GS_CONTEXT_DIALOG_ROW_IMPORTANCE_IMPORTANT:
 		icon_name = "dialog-warning-symbolic";
@@ -741,6 +754,7 @@ update_relations_list (GsHardwareSupportContextDialog *self)
 
 	gtk_widget_remove_css_class (self->lozenge, "green");
 	gtk_widget_remove_css_class (self->lozenge, "yellow");
+	gtk_widget_remove_css_class (self->lozenge, "orange");
 	gtk_widget_remove_css_class (self->lozenge, "red");
 	gtk_widget_remove_css_class (self->lozenge, "grey");
 
