@@ -5,7 +5,7 @@
  * Copyright (C) 2013 Matthias Clasen <mclasen@redhat.com>
  * Copyright (C) 2014-2018 Kalev Lember <klember@redhat.com>
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -89,7 +89,8 @@ gs_installed_page_get_app_section (GsApp *app)
 
 	if (state == GS_APP_STATE_INSTALLING ||
 	    state == GS_APP_STATE_QUEUED_FOR_INSTALL ||
-	    state == GS_APP_STATE_REMOVING)
+	    state == GS_APP_STATE_REMOVING ||
+	    state == GS_APP_STATE_DOWNLOADING)
 		return GS_UPDATE_LIST_SECTION_INSTALLING_AND_REMOVING;
 
 	if (kind == AS_COMPONENT_KIND_DESKTOP_APP) {
@@ -293,10 +294,11 @@ gs_installed_page_notify_state_changed_cb (GsApp *app,
 
 	gtk_list_box_row_changed (GTK_LIST_BOX_ROW (app_row));
 
-	/* Filter which applications can be shown in the installed page */
+	/* Filter which apps can be shown in the installed page */
 	if (state != GS_APP_STATE_INSTALLING &&
 	    state != GS_APP_STATE_INSTALLED &&
 	    state != GS_APP_STATE_REMOVING &&
+	    state != GS_APP_STATE_DOWNLOADING &&
 	    state != GS_APP_STATE_UPDATABLE &&
 	    state != GS_APP_STATE_UPDATABLE_LIVE)
 		gs_installed_page_unreveal_row (app_row);
@@ -334,7 +336,7 @@ gs_installed_page_add_app (GsInstalledPage *self, GsAppList *list, GsApp *app)
 {
 	GtkWidget *app_row;
 
-	/* only show if is an actual application */
+	/* only show if is an actual app */
 	if (!gs_installed_page_is_actual_app (app))
 		return;
 
@@ -557,11 +559,11 @@ gs_installed_page_switch_to (GsPage *page)
  *
  * Get a sort key to achive this:
  *
- * 1. state:installing applications
- * 2. state: applications queued for installing
- * 3. state:removing applications
- * 4. kind:normal applications
- * 5. kind:system applications
+ * 1. state:installing apps
+ * 2. state: apps queued for installing
+ * 3. state:removing apps
+ * 4. kind:normal apps
+ * 5. kind:system apps
  *
  * Within each of these groups, they are sorted by the install date and then
  * by name.
@@ -576,18 +578,21 @@ gs_installed_page_get_app_sort_key (GsApp *app)
 
 	/* sort installed, removing, other */
 	switch (gs_app_get_state (app)) {
-	case GS_APP_STATE_INSTALLING:
+	case GS_APP_STATE_DOWNLOADING:
 		g_string_append (key, "1:");
 		break;
-	case GS_APP_STATE_QUEUED_FOR_INSTALL:
+	case GS_APP_STATE_INSTALLING:
 		g_string_append (key, "2:");
 		break;
-	case GS_APP_STATE_REMOVING:
+	case GS_APP_STATE_QUEUED_FOR_INSTALL:
 		g_string_append (key, "3:");
 		break;
-	default:
+	case GS_APP_STATE_REMOVING:
 		g_string_append (key, "4:");
 		break;
+	default:
+		g_string_append (key, "5:");
+		break;
 	}
 
 	/* sort apps by kind */
@@ -733,7 +738,8 @@ gs_installed_page_pending_apps_refined_cb (GObject *source,
 									 res,
 									 &error);
 	if (list == NULL) {
-		if (!g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
+		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+		    !g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED))
 			g_warning ("failed to refine pending apps: %s", error->message);
 		return;
 	}
