@@ -47,7 +47,7 @@ struct _GsUpdatesPage
 	GsPluginLoader		*plugin_loader;
 	GCancellable		*cancellable;
 	GCancellable		*cancellable_refresh;
-	GCancellable		*cancellable_upgrade_download;
+	GCancellable		*cancellable_upgrade;
 	GSettings		*settings;
 	GSettings		*desktop_settings;
 	gboolean		 cache_valid;
@@ -149,7 +149,8 @@ _get_app_section (GsApp *app)
 
 	if (!gs_app_has_quirk (app, GS_APP_QUIRK_NEEDS_REBOOT) &&
 	    (gs_app_get_state (app) == GS_APP_STATE_UPDATABLE_LIVE ||
-	     gs_app_get_state (app) == GS_APP_STATE_INSTALLING)) {
+	     gs_app_get_state (app) == GS_APP_STATE_INSTALLING ||
+	     gs_app_get_state (app) == GS_APP_STATE_DOWNLOADING)) {
 		if (gs_app_get_kind (app) == AS_COMPONENT_KIND_FIRMWARE)
 			return GS_UPDATES_SECTION_KIND_ONLINE_FIRMWARE;
 		return GS_UPDATES_SECTION_KIND_ONLINE;
@@ -179,7 +180,8 @@ _get_num_updates (GsUpdatesPage *self)
 	for (guint i = 0; i < gs_app_list_length (apps); ++i) {
 		GsApp *app = gs_app_list_index (apps, i);
 		if (gs_app_is_updatable (app) ||
-		    gs_app_get_state (app) == GS_APP_STATE_INSTALLING)
+		    gs_app_get_state (app) == GS_APP_STATE_INSTALLING ||
+		    gs_app_get_state (app) == GS_APP_STATE_DOWNLOADING)
 			++count;
 	}
 	return count;
@@ -513,16 +515,19 @@ gs_updates_page_get_upgrades_cb (GObject *source_object,
 typedef struct {
 	GsApp		*app; /* (owned) */
 	GsUpdatesPage	*self; /* (owned) */
+	GsPluginJob	*job; /* (owned) */
 } GsPageHelper;
 
 static GsPageHelper *
 gs_page_helper_new (GsUpdatesPage *self,
-		    GsApp	 *app)
+		    GsApp	  *app,
+		    GsPluginJob   *job)
 {
 	GsPageHelper *helper;
 	helper = g_slice_new0 (GsPageHelper);
 	helper->self = g_object_ref (self);
 	helper->app = g_object_ref (app);
+	helper->job = g_object_ref (job);
 	return helper;
 }
 
@@ -530,6 +535,7 @@ static void
 gs_page_helper_free (GsPageHelper *helper)
 {
 	g_clear_object (&helper->app);
+	g_clear_object (&helper->job);
 	g_clear_object (&helper->self);
 	g_slice_free (GsPageHelper, helper);
 }
@@ -604,12 +610,12 @@ gs_updates_page_get_system_finished_cb (GObject *source_object,
 
 	refine_flags = GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE |
-		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS |
+		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_SEVERITY |
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION;
 
-	helper = gs_page_helper_new (self, app);
 	plugin_job = gs_plugin_job_refine_new_for_app (app, refine_flags);
 	gs_plugin_job_set_interactive (plugin_job, TRUE);
+	helper = gs_page_helper_new (self, app, plugin_job);
 	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
 					    self->cancellable,
 					    gs_updates_page_refine_system_finished_cb,
@@ -620,7 +626,6 @@ static void
 gs_updates_page_load (GsUpdatesPage *self)
 {
 	guint64 refine_flags;
-	g_autoptr(GsApp) app = NULL;
 	g_autoptr(GsPluginJob) plugin_job = NULL;
 
 	if (self->action_cnt > 0)
@@ -632,7 +637,7 @@ gs_updates_page_load (GsUpdatesPage *self)
 
 	refine_flags = GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE |
-		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS |
+		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_SEVERITY |
 		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION;
 	gs_updates_page_set_state (self, GS_UPDATES_PAGE_STATE_ACTION_GET_UPDATES);
 	self->action_cnt++;
@@ -890,11 +895,24 @@ upgrade_download_finished_cb (GObject *source,
 	g_autoptr(GsPageHelper) helper = user_data;
 	g_autoptr(GError) error = NULL;
 
+	g_clear_object (&helper->self->cancellable_upgrade);
+
 	if (!gs_plugin_loader_job_action_finish (plugin_loader, res, &error)) {
 		if (g_error_matches (error, GS_PLUGIN_ERROR, GS_PLUGIN_ERROR_CANCELLED) ||
 		    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			return;
-		g_warning ("failed to upgrade-download: %s", error->message);
+		gs_plugin_loader_claim_job_error (plugin_loader,
+						  NULL,
+						  helper->job,
+						  error);
+	} else if (!gs_page_is_active_and_focused (GS_PAGE (helper->self))) {
+		g_autoptr(GNotification) notif = NULL;
+
+		notif = g_notification_new (_("Software Upgrades Downloaded"));
+		g_notification_set_body (notif, _("Software upgrades have been downloaded and are ready to be installed."));
+		g_notification_set_default_action_and_target (notif, "app.set-mode", "s", "updates");
+		/* last the notification for an hour */
+		gs_application_send_notification (GS_APPLICATION (g_application_get_default ()), "upgrades-downloaded", notif, 60);
 	}
 }
 
@@ -906,24 +924,25 @@ gs_updates_page_upgrade_download_cb (GsUpgradeBanner *upgrade_banner,
 	GsPageHelper *helper;
 	g_autoptr(GsPluginJob) plugin_job = NULL;
 
+	g_application_withdraw_notification (g_application_get_default (), "upgrades-downloaded");
+
 	app = gs_upgrade_banner_get_app (upgrade_banner);
 	if (app == NULL) {
 		g_warning ("no upgrade available to download");
 		return;
 	}
 
-	helper = gs_page_helper_new (self, app);
-
-	if (self->cancellable_upgrade_download != NULL)
-		g_object_unref (self->cancellable_upgrade_download);
-	self->cancellable_upgrade_download = g_cancellable_new ();
-	g_debug ("Starting upgrade download with cancellable %p", self->cancellable_upgrade_download);
+	g_clear_object (&self->cancellable_upgrade);
+	self->cancellable_upgrade = g_cancellable_new ();
+	g_debug ("Starting upgrade download with cancellable %p", self->cancellable_upgrade);
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_UPGRADE_DOWNLOAD,
 					 "interactive", TRUE,
 					 "app", app,
+					 "propagate-error", TRUE,
 					 NULL);
+	helper = gs_page_helper_new (self, app, plugin_job);
 	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
-					    self->cancellable_upgrade_download,
+					    self->cancellable_upgrade,
 					    upgrade_download_finished_cb,
 					    helper);
 }
@@ -982,6 +1001,8 @@ upgrade_trigger_finished_cb (GObject *source,
 	GsUpdatesPage *self = (GsUpdatesPage *) user_data;
 	g_autoptr(GError) error = NULL;
 
+	g_clear_object (&self->cancellable_upgrade);
+
 	/* get the results */
 	if (!gs_plugin_loader_job_action_finish (self->plugin_loader, res, &error)) {
 		g_warning ("Failed to trigger offline update: %s", error->message);
@@ -1004,12 +1025,15 @@ trigger_upgrade (GsUpdatesPage *self)
 		return;
 	}
 
+	g_clear_object (&self->cancellable_upgrade);
+	self->cancellable_upgrade = g_cancellable_new ();
+
 	plugin_job = gs_plugin_job_newv (GS_PLUGIN_ACTION_UPGRADE_TRIGGER,
 					 "interactive", TRUE,
 					 "app", upgrade,
 					 NULL);
 	gs_plugin_loader_job_process_async (self->plugin_loader, plugin_job,
-					    self->cancellable,
+					    self->cancellable_upgrade,
 					    upgrade_trigger_finished_cb,
 					    self);
 }
@@ -1100,6 +1124,7 @@ gs_shell_update_are_updates_in_progress (GsUpdatesPage *self)
 		switch (gs_app_get_state (app)) {
 		case GS_APP_STATE_INSTALLING:
 		case GS_APP_STATE_REMOVING:
+		case GS_APP_STATE_DOWNLOADING:
 			return TRUE;
 			break;
 		default:
@@ -1133,21 +1158,6 @@ gs_updates_page_status_changed_cb (GsPluginLoader *plugin_loader,
                                    GsPluginStatus status,
                                    GsUpdatesPage *self)
 {
-	switch (status) {
-	case GS_PLUGIN_STATUS_INSTALLING:
-	case GS_PLUGIN_STATUS_REMOVING:
-		if (app == NULL ||
-		    (gs_app_get_kind (app) != AS_COMPONENT_KIND_OPERATING_SYSTEM &&
-		     gs_app_get_id (app) != NULL)) {
-			/* if we do a install or remove then make sure all new
-			 * packages are downloaded */
-			gs_updates_page_invalidate_downloaded_upgrade (self);
-		}
-		break;
-	default:
-		break;
-	}
-
 	gs_updates_page_update_ui_state (self);
 }
 
@@ -1167,8 +1177,8 @@ static void
 gs_updates_page_upgrade_cancel_cb (GsUpgradeBanner *upgrade_banner,
                                    GsUpdatesPage *self)
 {
-	g_debug ("Cancelling upgrade download with %p", self->cancellable_upgrade_download);
-	g_cancellable_cancel (self->cancellable_upgrade_download);
+	g_debug ("Cancelling upgrade with %p", self->cancellable_upgrade);
+	g_cancellable_cancel (self->cancellable_upgrade);
 }
 
 static gboolean
@@ -1314,8 +1324,8 @@ gs_updates_page_dispose (GObject *object)
 
 	g_cancellable_cancel (self->cancellable_refresh);
 	g_clear_object (&self->cancellable_refresh);
-	g_cancellable_cancel (self->cancellable_upgrade_download);
-	g_clear_object (&self->cancellable_upgrade_download);
+	g_cancellable_cancel (self->cancellable_upgrade);
+	g_clear_object (&self->cancellable_upgrade);
 
 	for (guint i = 0; i < GS_UPDATES_SECTION_KIND_LAST; i++) {
 		if (self->sections[i] != NULL) {
